#!/bin/bash

#   install_lamp.sh - Install a LAMP server
#
#   DISS - Debian Install Server Scripts
#   A set of scripts to automate installation of Servers on Debian
#   (c) 2009 Emmanuel Revah - manu-at-manurevah.com
#
#   This file is part of DISS.
#
#   DISS is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   DISS is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with DISS.  If not, see <http://www.gnu.org/licenses/>.


clear
source /etc/diss/diss.conf
source ${SCRIPTS}/functions.sh

THISSCRIPT=install_lamp.sh
THISSTATE=${WEBSTATE}
# test if this has already been done :
installStatus

# test if install_base has been done, if not show a message
DEPENDSTATE=${BASESTATE}
DEPENDSON="install_base"
dependsOn

echo "

#########################################
#					#
#	Web server installation		#
#					#
#########################################

This script will install:
 - Apache
 - Mysql
 - PHP5
 - Proftpd
 - Awstats
 - Munin
 - PhpSysinfo

Press enter to continue  =>
"
read

# set mysql root pass
getInpute "Set mysql *ROOT* password:" pw
SQLPASS="${ANSWER}"

# get username, main domain, admin subdomain and pass
echo "
This user will also be the main admin user"
getInpute "Enter user/client login:" user
CLIENT="${ANSWER}"

getInpute "Enter domain/site name (ex domain.tld):" domain
DOMAIN="${ANSWER}"

testDomain ${DOMAIN}

echo -n "Enter subdomain or alias (e.g. www.domain.tld or otherdomain.tld) OPTIONAL : "
read -e ALIAS
test ${ALIAS} || ALIAS="--None--"
if [ "${ALIAS}" != "--None--" ]; then
	testDomain ${ALIAS}
fi

echo "
We will create a subdomain for accessing Phpmyadmin, Awstats and system information."
getInpute "Subdomain (for https://private.${DOMAIN} enter \"private\"):" domain
NIMDA="${ANSWER}"
testDomain ${NIMDA}

getInpute "Password for ${CLIENT} (will be used for FTP, MySQL and web https://${NIMDA}.${DOMAIN}):" pw
CLIENTPASS="${ANSWER}"

# confirm data
echo "confirm the following before proceeding :

 main account name : ${CLIENT}
 main site : ${DOMAIN}"
if [ "${ALIAS}" = "0" ]; then
        echo " alias : --None--"
else
        echo " alias : ${ALIAS}"
fi
echo " sub domain for administration : ${NIMDA}.${DOMAIN}
 password : ${CLIENTPASS}
 root MySQL password : ${SQLPASS}

Double check the information above"
# confirm before actually doing anything
confirmORdie

# test user/client
test -d /home/www/html/${CLIENT} || NEW=1
if [ "$NEW" = "1" ]; then
	echo " => the user doesn't exist in DISS (good news)"
else
	echo " => the client already exists. .. .  must die()
	(check your configuration and reload script)"
	read
	exit
fi

# test if the user already exists on the system
egrep "^${CLIENT}" /etc/passwd >/dev/null
	if [ $? -eq 0 ]; then
		echo " => the user ${CLIENT} exists, we shall not recreate it NOR modify
the system password (used by FTP)."
		CREATEUSER=0
	else
		echo " => the user ${CLIENT} does not exist .. . . .. YET :]"
		CREATEUSER=1
fi

# adduser if needed
if [ "${CREATEUSER}" == "1" ]; then
	CLIENTNIXPASS=$(perl -e 'print crypt($ARGV[0], "password")' ${CLIENTPASS})
	useradd --shell /bin/false -p ${CLIENTNIXPASS} ${CLIENT} -d /home/www/html/${CLIENT}
fi

# create and prepare the client html dirs
mkdir -p /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}
mkdir -p /home/www/html/${CLIENT}/${DOMAIN}
cp /usr/local/${SOFTNAME}/nimda/index_nimda_global.php /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/index.php
cp -r /usr/local/${SOFTNAME}/nimda/style/ /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/
cp /usr/local/${SOFTNAME}/nimda/favicon.ico /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/
cp /usr/local/${SOFTNAME}/nimda/clients.php /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/
mkdir -p /var/log/apache2/${CLIENT}
chown -R root:www-data /home/www/html/${CLIENT}
chown ${CLIENT}:www-data /home/www/html/${CLIENT}/${DOMAIN}
chmod 750 /home/www/html/${CLIENT}/${DOMAIN}
chmod 750 /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}

# this stuff is for data that should not be in the http_dir
# you can use it to store web data that needs a little extra protection
# or needs a "home" such as squirrelmail data, svn projects, gallery etc.
mkdir -p /home/www/outside-html/vmail_manager
chown root:www-data /home/www/outside-html/
chmod 750 /home/www/outside-html/


# actually install and create/mv/etc files and stuff

# apt get the stuff
DEBIAN_PRIORITY=critical apt-get install -y mysql-server phpmyadmin proftpd hddtemp
aptitude install -y lha unrar openssl apache2 apache2-suexec libapache2-mod-php5 php5-mysql telnet php-pear php5-cli \
php5-curl php5 libapache2-mod-fcgid php5-cgi lftp ncftp munin munin-node phpsysinfo lm-sensors awstats

# create htpasswd file
htpasswd -cmb /home/www/.htpasswd ${CLIENT} ${CLIENTPASS}
chown root:www-data /home/www/.htpasswd
chmod 640 /home/www/.htpasswd

# set the MYSQL root passwd
mysqladmin password ${SQLPASS}

# create ssl certificate for apache
clear
echo "We shall make an Apache certificate for accessing
https://${NIMDA}.${DOMAIN}

to avoid \"domain mismatch\" errors make sure to answer
${NIMDA}.${DOMAIN} to the question \"common name\"

press enter when ready"
read
openssl req $@ -new -x509 -days 730 -nodes -out /etc/apache2/apache.pem -keyout /etc/apache2/apache.pem
chmod 600 /etc/apache2/apache.pem
echo "NameVirtualHost *:443" >> /etc/apache2/ports.conf
a2enmod ssl

# apache modules
a2enmod rewrite
a2enmod suexec
a2enmod include
a2enmod fcgid
a2enmod php5

# apache logs config
echo "/var/log/apache2/*/*.log {
    weekly
    missingok
    rotate 52
    compress
    delaycompress
    notifempty
    create 640 root adm
    sharedscripts
    postrotate
        if [ -f "`. /etc/apache2/envvars ; echo ${APACHE_PID_FILE:-/var/run/apache2.pid}`" ]; then
            /etc/init.d/apache2 reload > /dev/null
        fi
    endscript
}" >> /etc/logrotate.d/apache2


# configure sensors
clear
echo "

The correct answer to the next set of questions
should be \"YES\" however you may try to read and make
some sense of it.

This is for munin sensors plugin, so it's optional.

Press enter when ready
"
read
sensors-detect
modprobe w83627hf
cd /etc/munin/plugins/
ln -s /usr/share/munin/plugins/sensors_ sensors_temp
ln -s /usr/share/munin/plugins/sensors_ sensors_fan
ln -s /usr/share/munin/plugins/sensors_ sensors_volt
/etc/init.d/munin-node restart

# replace and modify some conf files
# munin and phpsysinfo
cp /etc/munin/munin.conf /etc/munin/munin.conf.diss-BKP
echo "dbdir   /var/lib/munin
htmldir /var/www/munin
logdir  /var/log/munin
rundir  /var/run/munin
tmpldir /etc/munin/templates

[${NIMDA}.${DOMAIN}]
    address 127.0.0.1
    use_node_name yes" > /etc/munin/munin.conf
/etc/init.d/munin-node restart

cd /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}
ln -s /var/www/munin .
ln -s /var/www/phpsysinfo .
ln -s /usr/share/phpmyadmin phashp
ln -s /home/www/outside-html/vmail_manager vmail_manager

# php and apache
cp /etc/php5/apache2/php.ini /etc/php5/apache2/php.ini.diss-BKP
cp ${DEFCONFS}/php/php.ini /etc/php5/apache2/php.ini
cp ${DEFCONFS}/php/php_cgi.ini /etc/php5/cgi/php.ini
cp /etc/apache2/apache2.conf /etc/apache2/apache2.conf.diss-BKP
cp ${DEFCONFS}/apache/apache2.conf /etc/apache2/apache2.conf
cp ${DEFCONFS}/apache/default /etc/apache2/sites-available/

# make webspace for default vhost
mkdir -p /home/www/html/default
cp -r ${DEFCONFS}/apache/default-www/* /home/www/html/default/

cp /etc/proftpd/proftpd.conf /etc/proftpd/proftpd.conf.diss-BKP
# main user doesn't need ftp access to everyones html_dir
#sed "s/replacedbyscript/${CLIENT}/" < ${DEFCONFS}/proftpd.conf > /etc/proftpd/proftpd.conf
sed "s/replacedbyscript/${FQDN}/" < ${DEFCONFS}/proftpd.conf > /etc/proftpd/proftpd.conf
# if main client/user already exists he will be badly ftprooted, so we add something
if [ CREATEUSER=0 ]; then
	sed -i "s#DefaultRoot ~#DefaultRoot /home/www/html/${CLIENT} ${CLIENT}#" /etc/proftpd/proftpd.conf
	echo "DefaultRoot ~" >> /etc/proftpd/proftpd.conf
fi

# update diss.conf
sed -i "s/ADMINUSER=/ADMINUSER=\"${CLIENT}\"/" ${DISSETC}/diss.conf
sed -i "s/ADMINSITE=/ADMINSITE=\"${DOMAIN}\"/" ${DISSETC}/diss.conf
sed -i "s/ADMINNIMDA=/ADMINNIMDA=\"${NIMDA}\"/" ${DISSETC}/diss.conf
sed -i "s/ADMINNIMDAURL=/ADMINNIMDAURL=\"${NIMDA}.${DOMAIN}\"/" ${DISSETC}/diss.conf
sed -i "s/ADMINPATH=/ADMINPATH=\"\/home\/www\/html\/${CLIENT}\"/" ${DISSETC}/diss.conf
sed -i "s/ADMINNIMDAPATH=/ADMINNIMDAPATH=\"\/home\/www\/html\/${CLIENT}\/${NIMDA}.${DOMAIN}\"/" ${DISSETC}/diss.conf
sed -i "s/ROOTSQL=/ROOTSQL=\"${SQLPASS}\"/" ${DISSETC}/diss.conf

if [ "${ALIAS}" != "--None--" ]; then
	CLIENTDOMAIN="${DOMAIN} ${ALIAS}"
else
	CLIENTDOMAIN="${DOMAIN}"
fi

# DISS per client config file
echo "#!/bin/bash

# configuration for the client ${CLIENT}
CLIENT=${CLIENT}
CLIENTSITE=${DOMAIN}
CLIENTNIMDA=${NIMDA}
CLIENTNIMDAURL=${NIMDA}.${DOMAIN}
CLIENTPATH=/home/www/html/${CLIENT}
CLIENTNIMDAPATH=/home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}
CLIENTPASS=${CLIENTPASS}
CLIENTDOMAIN[1]=\"${CLIENTDOMAIN}\"
"  > /etc/${SOFTNAME}/clients/${CLIENT}.conf
chmod 600 /etc/${SOFTNAME}/clients/${CLIENT}.conf

# configuration vhosts apache
echo "<VirtualHost *:443>
SSLEngine on
SSLCertificateFile /etc/apache2/apache.pem

ServerName ${NIMDA}.${DOMAIN}
DocumentRoot /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}
CustomLog /var/log/apache2/${CLIENT}/${NIMDA}.${DOMAIN}.log combined
CustomLog /var/log/apache2/global.log combined

Alias /awstatsicons \"/usr/share/awstats/icon/\"
Alias /awstatscss \"/usr/share/doc/awstats/examples/css\"
Alias /awstatsclasses \"/usr/share/awstats/lib/\"
ScriptAlias /awstats/awstats.pl \"/usr/lib/cgi-bin/awstats.pl\"

<Directory /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/phashp>
	Options FollowSymLinks MultiViews
	AllowOverride None
	Order allow,deny
	allow from all
</Directory>

</VirtualHost>

<Files \"awstats.pl\">
	AuthUserFile /home/www/.htpasswd
	AuthGroupFile /dev/null
	AuthName \"Please your enter credentials\"
	AuthType Basic
	require valid-user
</Files>

<Directory /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}>
	AllowOverride AuthConfig
	Options FollowSymLinks MultiViews
	Order allow,deny
	allow from all
</Directory>
" > /etc/apache2/sites-available/${CLIENT}_${NIMDA}.${DOMAIN}

# add vhost configuration, with or without ServerAlias
if [ "${ALIAS}" = "--None--" ]; then
        echo "<VirtualHost *:80>
ServerName ${DOMAIN}
DocumentRoot /home/www/html/${CLIENT}/${DOMAIN}
SuexecUserGroup ${CLIENT} ${CLIENT}
<Directory /home/www/html/${CLIENT}/${DOMAIN}>
        Options +ExecCGI
        AllowOverride All
        AddHandler fcgid-script .php
        FCGIWrapper /var/www/php-fcgi-scripts/${CLIENT}/php-fcgi-starter .php
        Order allow,deny
        Allow from all
</Directory>
CustomLog /var/log/apache2/${CLIENT}/${DOMAIN}.log combined
CustomLog /var/log/apache2/global.log combined
</VirtualHost>" > /etc/apache2/sites-available/${CLIENT}_${DOMAIN}
else

echo "<VirtualHost *:80>
ServerName ${DOMAIN}
ServerAlias ${ALIAS}
DocumentRoot /home/www/html/${CLIENT}/${DOMAIN}
SuexecUserGroup ${CLIENT} ${CLIENT}
<Directory /home/www/html/${CLIENT}/${DOMAIN}>
        Options +ExecCGI
        AllowOverride All
        AddHandler fcgid-script .php
        FCGIWrapper /var/www/php-fcgi-scripts/${CLIENT}/php-fcgi-starter .php
        Order allow,deny
        Allow from all
</Directory>
CustomLog /var/log/apache2/${CLIENT}/${DOMAIN}.log combined
CustomLog /var/log/apache2/global.log combined
</VirtualHost>" > /etc/apache2/sites-available/${CLIENT}_${DOMAIN}
fi

# add a configuration example
echo "<VirtualHost *:80>
ServerName example.com
ServerAlias www.example.com
DocumentRoot /home/www/html/example-client/example.com
CustomLog /home/www/html/example-client/log/apache2/example.com.log combined
CustomLog /var/log/apache2/global.log combined
</VirtualHost>" > /etc/apache2/sites-available/example

# create htaccess
echo "AuthUserFile /home/www/.htpasswd
AuthName \"Enter Credentials\"
AuthType Basic

<Limit GET POST>
Require user ${CLIENT}
</Limit>" > /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/.htaccess
chown root:www-data /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/.htaccess
chmod 640 /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/.htaccess

# create /var/www/php-fcgi-scripts/${CLIENT}/php-fcgi-starter
mkdir -p /var/www/php-fcgi-scripts/${CLIENT}
echo "#!/bin/sh
PHPRC=/etc/php5/cgi/
export PHPRC
export PHP_FCGI_MAX_REQUESTS=5000
export PHP_FCGI_CHILDREN=8
exec /usr/lib/cgi-bin/php" > /var/www/php-fcgi-scripts/${CLIENT}/php-fcgi-starter
chown -R ${CLIENT}:${CLIENT} /var/www/php-fcgi-scripts/${CLIENT}/
chmod 755 /var/www/php-fcgi-scripts/${CLIENT}/
chmod +x /var/www/php-fcgi-scripts/${CLIENT}/php-fcgi-starter


#####################
# Configure Awstats #
#####################

mkdir /var/cache/awstats
chown www-data:www-data /var/cache/awstats
mkdir /etc/awstats/diss-BKP/
mv /etc/awstats/awstats* /etc/awstats/diss-BKP/
cp ${DEFCONFS}/awstats/awstats.ftp.conf /etc/awstats/
cp ${DEFCONFS}/awstats/include.conf /etc/awstats/
cp ${DEFCONFS}/awstats/diss-logo_awstats.png /usr/share/awstats/icon/other/

# make web stats for ftp server usage
sed -i "s/SiteDomain=/SiteDomain=\"ftp.${DOMAIN}\"/" /etc/awstats/awstats.ftp.conf
sed -i "s/AllowAccessFromWebToFollowingAuthenticatedUsers=/AllowAccessFromWebToFollowingAuthenticatedUsers=\"${ADMINUSER}\"/" /etc/awstats/awstats.ftp.conf

# links awstats update script to /usr/local/sbin + cron it
ln -s /usr/local/diss/sbin/awstats-cronbin.sh /usr/local/sbin/
chmod +x /usr/local/sbin/awstats-cronbin.sh
echo "0 6,12,18,22 * * * root /usr/local/sbin/awstats-cronbin.sh &> /dev/null" >> /etc/crontab

# create conf files for domain, admin domain and global stats
echo  "LogFile=\"/var/log/apache2/${CLIENT}/${NIMDA}.${DOMAIN}.log\"
SiteDomain=\"${NIMDA}.${DOMAIN}\"
AllowAccessFromWebToAuthenticatedUsersOnly=1
AllowAccessFromWebToFollowingAuthenticatedUsers=\"${CLIENT}\"
DefaultFile=\"index.html\"
ShowFlagLinks=\"en de fr\"
Lang=\"en\"
Include \"include.conf\"" > /etc/awstats/awstats.${NIMDA}.${DOMAIN}.conf

if [ "${ALIAS}" = "--None--" ]; then
	echo  "LogFile=\"/var/log/apache2/${CLIENT}/${DOMAIN}.log\"
SiteDomain=\"${DOMAIN}\"
AllowAccessFromWebToAuthenticatedUsersOnly=1
AllowAccessFromWebToFollowingAuthenticatedUsers=\"${CLIENT}\"
DefaultFile=\"index.html\"
ShowFlagLinks=\"en de fr\"
Lang=\"en\"
Include \"include.conf\"" > /etc/awstats/awstats.${DOMAIN}.conf
else
	echo  "LogFile=\"/var/log/apache2/${CLIENT}/${DOMAIN}.log\"
SiteDomain=\"${DOMAIN}\"
HostAliases=\"${ALIAS}\"
AllowAccessFromWebToAuthenticatedUsersOnly=1
AllowAccessFromWebToFollowingAuthenticatedUsers=\"${CLIENT}\"
DefaultFile=\"index.html\"
ShowFlagLinks=\"en de fr\"
Lang=\"en\"
Include \"include.conf\"" > /etc/awstats/awstats.${DOMAIN}.conf
fi

echo  "LogFile=\"/var/log/apache2/global.log\"
SiteDomain=\"global\"
AllowAccessFromWebToAuthenticatedUsersOnly=1
AllowAccessFromWebToFollowingAuthenticatedUsers=\"${CLIENT}\"
DefaultFile=\"index.html\"
ShowFlagLinks=\"en de fr\"
Lang=\"en\"
Include \"include.conf\"" > /etc/awstats/awstats.global.conf
######## End Awstats conf ###########


# create links in admin page
echo "<h3>Main account:  ${CLIENT}</h3>
<ul>
	<li><a href=\"/awstats/awstats.pl?config=${DOMAIN}\">${DOMAIN}</a></li>
	<li><a href=\"/awstats/awstats.pl?config=${NIMDA}.${DOMAIN}\">${NIMDA}.${DOMAIN}</a></li>
" > /home/www/html/${CLIENT}/${NIMDA}.${DOMAIN}/links.html

a2ensite ${CLIENT}_${NIMDA}.${DOMAIN}
a2ensite ${CLIENT}_${DOMAIN}

echo "testing apache configuration

apache2ctl configtest: "
echo "`apache2ctl configtest`"

echo "press enter to restart apache and ftp and also to continue <="
read

/etc/init.d/apache2 restart
/etc/init.d/proftpd restart

# update install status in diss.conf and open required ports
sed -i "s/WEBSTATE=\"not installed\"/WEBSTATE=\"done\"/" ${DISSETC}/diss.conf

# firewall stuff - add new ports to diss.conf and restart firewall it is installed
NEWTCPPORTS="${ARNO_OPEN_TCP} 21 80 443"
sed -i "s/ARNO_OPEN_TCP=\"${ARNO_OPEN_TCP}\"/ARNO_OPEN_TCP=\"${NEWTCPPORTS}\"/" ${DISSETC}/diss.conf
echo "
DISS: updating open ports (${NEWTCPPORTS}) in firewall config"
if [ -a "/etc/arno-iptables-firewall/debconf.cfg" ]; then
	source /etc/arno-iptables-firewall/debconf.cfg
	sed -i "s/DC_OPEN_TCP=\"${DC_OPEN_TCP}\"/DC_OPEN_TCP=\"${NEWTCPPORTS}\"/" /etc/arno-iptables-firewall/debconf.cfg
	/etc/init.d/arno-iptables-firewall restart
fi

echo "
End of installation, we made it !!

press enter to get a print out of what you need to remember
  <=
"
read
clear


echo "

########################
#                      #
#	INFOS	       #
#                      #
########################

WEB/FTP
- Main user account (FTP): ${CLIENT}
- Password for user ${CLIENT}: ${CLIENTPASS}
- Configured Web space for: ${DOMAIN}
- Absolute path: /home/www/html/${CLIENT}/${DOMAIN}

DB/PhpMyadmin
- Root MySQL user is: root
- Password for MySQL user 'root': ${SQLPASS}

DNS stuff
- Server's IP is: ${IP}
- DNS (Bind) configuration should look something like:
        ${NIMDA}.${DOMAIN} 3600 IN A ${IP}
        ${DOMAIN} 3600 IN A ${IP}" > /etc/diss/infos_lamp
if [ "${ALIAS}" != "--None--" ]; then
        echo "	${ALIAS}  3600 IN A  ${IP}" >> /etc/diss/infos_lamp
fi

echo "
- A bit more simple (depends on how you manage your DNS):
        ${NIMDA}.${DOMAIN}  A  ${IP}
        ${DOMAIN}  A  ${IP}" >> /etc/diss/infos_lamp
if [ "${ALIAS}" != "--None--" ]; then
        echo "	${ALIAS}  A  ${IP}" >> /etc/diss/infos_lamp
fi

echo "
Administration and misc:
- To manage and view PhpMyadmin and Awstats:
	https://${NIMDA}.${DOMAIN}
	or https://${IP}/
	User: ${CLIENT}
	Password: ${CLIENTPASS}

- The FTP account should arrive directly in /home/www/${CLIENT}" >> /etc/diss/infos_lamp
chmod 600 /etc/diss/infos_lamp
echo "`cat /etc/diss/infos_lamp`

press a button to exit the Web_Server_install and go back to the main menu"
read
exit
